#!/usr/bin/env node

const axios = require('axios');
const endpoint = 'https://en.wikipedia.org/w/rest.php/v1/search/page';
const chalk = require('chalk');
const process = require('process');

const printBoxedText = (text) => {
  const terminalWidth = process.stdout.columns;
  const maxWidth = Math.floor(terminalWidth * 0.55) - 4;
  const lines = text.split('_-__-_').flatMap(segment => {
    const segmentLines = [];
    let currentLine = '';

    segment.split(' ').forEach(word => {
      if ((currentLine + ' ' + word).trim().length > maxWidth) {
        if (['.', ','].includes(word[0])) {
          currentLine += word[0];
          word = word.substring(1);
        }
        segmentLines.push(currentLine.trim());
        currentLine = word;
      } else {
        currentLine += ' ' + word;
      }
    });

    if (currentLine.trim()) {
      segmentLines.push(currentLine.trim());
    }

    return [...segmentLines, '', ''];
  }).slice(0, -2);

  const maxLength = Math.max(...lines.map(line => line.length));
  const topBorder = chalk.bgBlack.white('┌' + '─'.repeat(maxLength + 2) + '┐');
  const bottomBorder = chalk.bgBlack.white('└' + '─'.repeat(maxLength + 2) + '┘');
  const blankLine = chalk.bgBlack.white('│' + ' '.repeat(maxLength + 2) + '│');

  console.log(`\n${topBorder}`);
  console.log(blankLine);
  lines.forEach(line => {
    const padding = ' '.repeat(maxLength - line.length);
    console.log(line ? (chalk.bgBlack.white('│') + chalk.bgBlack.yellow(' ' + line + padding + ' ') + chalk.bgBlack.white('│')) : blankLine);
  });
  console.log(blankLine);
  console.log(`${bottomBorder}\n`);
}

const search = async (query, limit) => {
  const spinner = require('ora')('Searching...').start();
  let success = false, res = {};

  while (!success) {
    res = await axios.get(endpoint, {
      params: {
        q: query,
        limit: limit || 1
      }
    });
    if (res.status === 200) success = true;
  }

  let pages = res.data.pages, final, segments = [];

  if (pages.length >= 1) {
    pages.forEach(page => {
      segments.push(page?.excerpt?.split('<span class="searchmatch">').join('').split('</span>').join(''));
    });
    final = segments.join('_-__-_');
  } else {
    final = 'No results found';
  }

  spinner.stop();
  return final;
}

if (require.main === module) {
  search(process.argv[2], process.argv[3] || 1).then(res => {
    printBoxedText(res);
  });
}

module.exports = { search, printBoxedText };
